/*

  Simple C-like dynamic vectors/matrices

  use doble, no typedef no template

*/

#include <cstddef>

// if C++ < C++11 define nullptr
#if __cplusplus <= 199711L
  #include <cstdlib>
  #ifndef nullptr
    #include <cstddef>
    #define nullptr NULL
  #endif
#endif

// define a vector as a struct with the size of the vector
// and a pointer to its values

struct Vec {
  int      size ;
  double * values ;

  // methods
  void Initialize() ;
  void Allocate( int size ) ;
  void Fill( double val ) ;
  void Free( ) ;

  // magic methods, redefine operators!
  double
  operator () // name of the operator
  ( int i ) const // const is used to inform the compiler that operator do not modifty contents
  { return values[i] ; }   

  double & // return the reference (i.e. the address) of values[i]
  operator () ( int i )
  { return values[i] ; }   

} ;

struct Mat {
  int       nrow, ncol ;
  double  * memory_block ; // allocated memory for the matrix
  double ** values ; // pointer to the rows of the matrix

  void Initialize( ) ;
  void Allocate( int nrow, int ncol ) ;
  void AllocateAndInitializeRandom( int nrow, int ncol ) ;
  void Fill( double val ) ;
  void Free( ) ;

  // magic methods, redefine operators!
  double
  operator () // name of the operator
  ( int i, int j ) const ; // const is used to inform the compiler that operator do not modifty contents

  double & // return the reference (i.e. the address) of values[i]
  operator () ( int i, int j ) ;

} ;

// magic methods, redefine operators!
inline
double
Mat::operator () // name of the operator
( int i, int j ) const // const is used to inform the compiler that operator do not modifty contents
{ return values[i][j] ; }   

inline
double & // return the reference (i.e. the address) of values[i]
Mat::operator () ( int i, int j )
{ return values[i][j] ; }   


void
Mat_x_Vec( struct Mat const & mat, // ensure the compiler that mat is not modified inside...
           struct Vec const & vec,
           struct Vec       & res ) ;

void
Mat_x_Mat_123( struct Mat const & A, 
               struct Mat const & B,
               struct Mat       & C ) ;
void
Mat_x_Mat_132( struct Mat const & A, 
               struct Mat const & B,
               struct Mat       & C ) ;
void
Mat_x_Mat_213( struct Mat const & A, 
               struct Mat const & B,
               struct Mat       & C ) ;
void
Mat_x_Mat_231( struct Mat const & A, 
               struct Mat const & B,
               struct Mat       & C ) ;
void
Mat_x_Mat_312( struct Mat const & A, 
               struct Mat const & B,
               struct Mat       & C ) ;
void
Mat_x_Mat_321( struct Mat const & A, 
               struct Mat const & B,
               struct Mat       & C ) ;

void
Mat_x_Mat_with_tiling( struct Mat const & A, 
                       struct Mat const & B,
                       struct Mat       & C,
                       int tsize ) ;
